#  bundle.rb
#  
#  Created by Keith Duncan on 21/10/2011.
#  Copyright (c) 2011 Realmac Software. All rights reserved.

class Bundle
	
	def self.childBundleLocationsOfBundleAtLocation(bundleLocation = nil)
		searchLocations = []
		searchLocations << locationForBundleAtURL(bundleLocation, "SharedSupport")
		searchLocations << locationForBundleAtURL(bundleLocation, "PlugIns")
		searchLocations << locationForBundleAtURL(bundleLocation, "RMFrameworks")
		searchLocations << locationForBundleAtURL(bundleLocation, "Library/QuickLook")
		searchLocations << locationForBundleAtURL(bundleLocation, "Library/Spotlight")
		searchLocations << locationForBundleAtURL(bundleLocation, "Library/LoginItems")
		searchLocations << supportLocationForBundleAtURL(bundleLocation)
		searchLocations << xpcServicesLocationForBundleAtURL(bundleLocation)
		searchLocations.concat(frameworkDirectoryLocationsForBundleAtURL(bundleLocation))
		
		childBundleLocations = []
		searchLocations.each do |currentSearchLocation|
			next unless Dir.exists? currentSearchLocation
			Dir.foreach(currentSearchLocation) do |childBundleLocation|
				next if childBundleLocation == '.' or childBundleLocation == '..'
				next if !itemIsBundle childBundleLocation
				childBundleLocations << File.join(currentSearchLocation, childBundleLocation)
			end
		end
		
		childBundleLocations.each do |currentBundleLocation|
			childBundleLocations = childBundleLocations + childBundleLocationsOfBundleAtLocation(currentBundleLocation)
		end
		
		childBundleLocations
	end
	
	def self.contentsDirectoryForBundleAtURL(bundleLocation)
		if File.extname(bundleLocation) == ".framework"
			return File.join(bundleLocation, "Versions", "Current")
		else
			return File.join(bundleLocation, "Contents")
		end
	end
	
	def self.locationForBundleAtURL(bundleLocation, directory)
		File.join(contentsDirectoryForBundleAtURL(bundleLocation), directory);
	end
	
	def self.libraryLocationForBundleAtURL(bundleLocation)
		locationForBundleAtURL(bundleLocation, "Library")
	end

	def self.supportLocationForBundleAtURL(bundleLocation)
		locationForBundleAtURL(bundleLocation, "Support")
	end
	
	def self.xpcServicesLocationForBundleAtURL(bundleLocation)
		locationForBundleAtURL(bundleLocation, "XPCServices")
	end

	def self.frameworkDirectoryLocationsForBundleAtURL(bundleLocation)
		[locationForBundleAtURL(bundleLocation, "Frameworks"), locationForBundleAtURL(bundleLocation, "SharedFrameworks")]
	end
	
	def self.itemIsBundle(item)
		candidateBundleExtensions = []
		candidateBundleExtensions << ".app"
		candidateBundleExtensions << ".framework"
		candidateBundleExtensions << ".xpc"
		candidateBundleExtensions << ".uploader"
		candidateBundleExtensions << ".dylib"
		candidateBundleExtensions << ".rwplugin"
		candidateBundleExtensions << ".rapidweaverplugin"
		candidateBundleExtensions << ".qlgenerator"
		candidateBundleExtensions << ".websnapper"
		candidateBundleExtensions << ".a"
		
		candidateBundleExtensions.include?(File.extname(item))
	end
	
end
