#!/System/Library/Frameworks/Ruby.framework/Versions/1.8/usr/bin/ruby

#  bundle_checklocalisations.rb
#  
#  Copyright (c) 2012 Realmac Software. All rights reserved.

require 'osx/cocoa'
include OSX

libraryDirectoryPath = File.dirname(__FILE__)
$LOAD_PATH << libraryDirectoryPath

def usage
	print "Usage:\n"
	print "-bundlePath: path to a bundle whose imageProj directories should be verified [required]\n"
	print "\n"
	print "Example:\n"
	print "ruby bundle_checkimages.rb\n -bundlePath ~/Projects/Products/Release\\ -\\ Standard/Courier.app\n"
	print "\n"
	exit 1
end

if ARGV.count == 0
	usage
end

def start
	print "=========    IMAGE VALIDATION STARTED     ==========\n"
	print "\n"
end

def finish
	print "\n"
	print "=========    IMAGE VALIDATION SUCCESS     ==========\n"
	exit 0
end

def fail(severity = :warning, reason = nil)
	if severity != :silent
		print "\n"
		print "=========    IMAGE VALIDATION FAILURE     ==========\n"
		unless reason.nil?
			print "error: #{reason}\n"
		end
		print "\n"
	end
	
	if severity == :fatal
		exit 1
	end
	
	def finish
		fail :fatal
	end
end


# Check arguments

bundlePath = NSUserDefaults.standardUserDefaults.stringForKey "bundlePath"
if bundlePath.nil?
	fail :fatal, "a bundle path must be specified using -bundlePath"
end
unless NSFileManager.defaultManager.fileExistsAtPath bundlePath
	fail :fatal, "bundle does not exist at the specified path"
end
bundleURL = NSURL.fileURLWithPath bundlePath
bundle = NSBundle.bundleWithURL bundleURL
if bundle.nil?
	fail :fatal, "couldn't load bundle"
end

bundleName = bundlePath.lastPathComponent


#

start


#

resourceURL = bundle.resourceURL
resourceRelativeImageProjPaths = NSFileManager.defaultManager.subpathsAtPath(resourceURL.path).pathsMatchingExtensions(NSArray.arrayWithObject("imageProj"))

def urlsByAppendingPaths(url, paths)
	urls = NSMutableArray.array
	paths.each do |currentPath|
		urls << url.URLByAppendingPathComponent(currentPath)
	end
	urls
end

imageProjLocations = urlsByAppendingPaths(resourceURL, resourceRelativeImageProjPaths)

checkScriptLocation = NSURL.fileURLWithPath(libraryDirectoryPath.to_ns).URLByAppendingPathComponent("imageProj_check.rb")

imageProjLocations.each do |currentImageProjLocation|
	next if currentImageProjLocation.lastPathComponent.stringByDeletingPathExtension.isEqualToString "snow_leopard"
	
	$stdout.flush
	checkTask = NSTask.launchedTaskWithLaunchPath_arguments_ checkScriptLocation.path, [ "-imageProjPath", currentImageProjLocation.path ]
	checkTask.waitUntilExit
	
	next if checkTask.terminationStatus == 0
	
	fail :silent
end


#

finish
