#!/bin/bash

set -e
set -o pipefail

argc="$#"
argv=("$@")

script_library_directory="$( cd "$( dirname "${BASH_SOURCE[0]}" )" && pwd )"

sourceRoot=""
force=0

while getopts ":-:s:o:" optchar
do
	case "${optchar}" in
		-)
			case "${OPTARG}" in
				sourceRoot)
					srcroot="${!OPTIND}"
					OPTIND=$(( $OPTIND + 1 ))
					;;
				force)
					force=1
					;;
				*)
					if [ "${OPTERR}" = 1 ] && [ "${optspec:0:1}" != ":" ]
					then
						echo "unknown option --${OPTARG}" >&2
						unknown_options=1
					fi
					;;
			esac
			;;
		s)
			srcroot="${OPTARG}"
			;;
		f)
			force=1
			;;
		*)
			if [ "${OPTERR}" = 1 ] && [ "${optspec:0:1}" != ":" ]
			then
				echo "unknown option --${OPTARG}" >&2
				unknown_options=1
			fi
			;;
	esac
done

if [ ! -d "${srcroot}" ]
then
	echo "--sourceRoot \"${srcroot}\" must exist and be a directory"
	exit -1
fi

if [ ! -z "${unknown_options}" ]
then
	exit -1
fi

# Utility Functions

function svnversion_stripped {
	workingcopy_path="${1}"
	svnversion "${workingcopy_path}" | tr -d '[:digit:]'
}

function check_workingcopy_revision_offer_recovery {
	current_workingcopy_path="${1}"
	
	echo -e "Checking ${current_workingcopy_path}"
	
	svnversion_string="$( svnversion ${current_workingcopy_path} )"
	if [ "${svnversion_string}" == "exported" ]
	then
		echo -e "> working copy is exported, skipping"
		return
	elif [ "${svnversion_string}" == "Unversioned directory" ]
	then
		echo -e "> working copy is unversioned, skipping"
		return
	fi
	
	if [ -z "$( svnversion_stripped ${current_workingcopy_path} )" ]
	then
		echo -e "> working copy is clean"
		return
	fi
	
	if [ ! $force -eq 1 ]
	then
		read -r -p "> working copy is a mixed revision, update source root before building? [y/n]: "
		if [ "${REPLY}" != "y" ]
		then
			echo -e "> user intervention, aborting" "\n"
			exit 1
		fi
	fi
	
	svn up --ignore-externals "${current_workingcopy_path}"

	if [ ! -z "$( svnversion_stripped ${current_workingcopy_path} )" ]
	then
		echo -e "> working copy remains a mixed revision after an update, aborting" "\n"
		exit 1
	fi
}

function main {
	# Check all folders for a round SVN revision
	
	declare -a directories
	IFS=$'\r\n' directories=($( find "${srcroot}" \( -type d -and -not -path '*.svn*' \) ))
	
	for current_directory in "${directories[@]}"
	do
		check_workingcopy_revision_offer_recovery "${current_directory}"
	done
}

main
