#!/System/Library/Frameworks/Ruby.framework/Versions/1.8/usr/bin/ruby

require 'osx/cocoa'
include OSX

script_library_directory = File.expand_path(File.dirname(__FILE__).to_ns).to_ns
$LOAD_PATH << script_library_directory

def usage
	print "Usage:\n"
	print "-sourceRoot: path to the project source root"
	print "-workingCopyPaths: ASCII encoded PLIST array of arrays containing first an array of directories to search for NSLocalizedString references and second the path to the genstrings output directory"
	print "\n"
	exit 1
end

if ARGV.count == 0
	usage
end

def finish
	print "============  GENSTRINGS SUCCESS   ============\n"
end

def fail(reason = nil, severity = :warning)
	print "\n"
	print "============  GENSTRINGS FAILURE   ============\n"
	unless reason.nil?
		print "error: #{reason}\n"
	end
	print "\n"
	
	if severity == :fatal
		exit 1
	end
	
	def finish
		fail nil, :fatal
	end
end

# Check arguments

source_root = NSUserDefaults.standardUserDefaults.stringForKey("sourceRoot")
if source_root.nil?
	fail "the source root must be specified using -sourceRoot", :fatal
end
source_root_location = NSURL.fileURLWithPath source_root

genstrings_wrapper_path = script_library_directory.stringByAppendingPathComponent("genstrings.sh")
unless NSFileManager.defaultManager.fileExistsAtPath genstrings_wrapper_path
	fail "couldn't find the genstrings wrapper", :fatal
end

working_copy_paths = NSUserDefaults.standardUserDefaults.arrayForKey("workingCopyPaths")
if working_copy_paths.nil?
	fail "the working copy paths must be specified using -workingCopyPaths", :fatal
end

# Invoke the wrapper for each path combination

working_copy_paths.each do |current_tuple|
	current_failure = lambda do
		fail "workingCopyPaths entry doesn't conform to schema, entry is #{current_tuple}", :fatal
	end
	current_failure.call unless current_tuple.isKindOfClass NSArray.objc_send(:class)
	current_failure.call unless current_tuple.count == 2
	
	all_paths = []
	
	sources_array = current_tuple.objectAtIndex 0
	current_failure.call unless sources_array.isKindOfClass NSArray.objc_send(:class)
	sources_array.each do |current_source_path|
		current_failure.call unless current_source_path.isKindOfClass NSString.objc_send(:class)
		
		unless current_source_path.hasPrefix "/"
			current_source_path = source_root.stringByAppendingPathComponent current_source_path
		end
		current_source_path_subpaths_or_error = NSFileManager.defaultManager.subpathsOfDirectoryAtPath_error(current_source_path)
		if current_source_path_subpaths_or_error[0].nil?
			error = current_source_path_subpaths_or_error[1]
			fail "couldn't read subpaths of #{current_source_path}, #{error.localizedDescription}"
			next
		end
		current_source_path_subpaths = current_source_path_subpaths_or_error[0]
		
		all_paths.concat current_source_path_subpaths
	end
	
	output_path = current_tuple.objectAtIndex 1
	current_failure.call unless output_path.isKindOfClass NSString.objc_send(:class)
	unless output_path.hasPrefix "/"
		output_path = source_root.stringByAppendingPathComponent output_path
	end
	
	print "Generating strings for #{output_path}\n"
	
	all_paths = all_paths.to_ns.pathsMatchingExtensions [ "h", "m", "c" ]
	
	task = NSTask.alloc.init
	task.setLaunchPath "/usr/bin/genstrings"
	
	arguments = [ "-q", "-u" ]
	arguments.concat [ "-o", output_path ]
	arguments.concat [ "-s", "RMLocalisedString" ]
	arguments.concat all_paths
	task.setArguments arguments.to_ns
	
	print "#{task.launchPath} #{task.arguments.componentsJoinedByString(' ')}\n"
	
	$stdout.flush
	task.launch
	task.waitUntilExit
	
	unless task.terminationStatus == 0
		fail "couldn't generate strings for #{current_tuple}"
	end
	
	print "\n"
end

#

finish
